#!/bin/bash
source ../tools/functions.sh

# Check if needed commands exist
abortIfCommandNotExists openssl "openssl not found, please install it before running kiuwan-certool.sh"
abortIfCommandNotExists keytool "keytool not found, please install a compatible Java Runtime Environment before running kiuwan-certool.sh"

VOLUMES_PROPERTIES_FILE="../config/volumes.properties"
CONFIG_SHARED_PATH=$(getProperty "config.shared" $VOLUMES_PROPERTIES_FILE)
abortIfDirNotFound $CONFIG_SHARED_PATH
GLOBAL_CONFIG_PATH=$CONFIG_SHARED_PATH/globalConfig/globalConfig.properties
abortIfNotFound $GLOBAL_CONFIG_PATH

# Kiuwan domain parameter
kiuwan_domain="$1"

# Output files
config_file=config/certs.properties
ca_crt_file=ca/cacert.pem
ca_key_file=ca/cakey.pem
key_file=${kiuwan_domain}/domainkey.pem
csr_file=${kiuwan_domain}/domain.csr
crt_file=${kiuwan_domain}/domaincert.pem
p12_file=${kiuwan_domain}/domain.p12
jks_file=${kiuwan_domain}/domainkeystore.jks
truststore_file=${kiuwan_domain}/truststore.jks
default_truststore_file=truststore/truststore.jks

CA_KEY_EXISTS="true"
if [ -f "$ca_key_file" ]
then
	CA_KEY_EXISTS="true"
	echo "CA KEY exists"
else
	CA_KEY_EXISTS="false"
	echo "CA KEY not exists"
fi
echo "CA_KEY_EXISTS=$CA_KEY_EXISTS"
# Parameters validation
if [ -z "$1" ] ; then
	echo "kiuwan-certool - generates test certificates and java stores"
	echo "	for the specified domain using the configuration set in "
	echo "	config/certs.properties"
	echo
	echo "ERROR: please specify a kiuwan domain when calling this script, e.g.:"
	echo "./kiuwan-certool.sh kiuwan.mycompany.com"
	exit 1
fi

echo "Checking if certs.properties file is found..."
abortIfNotFound "${config_file}"

# Read plain properties
java_keystore_password=$(getProperty "java.keystore.password" ${config_file})
java_truststore_password=$(getProperty "java.truststore.password" ${config_file})
ssl_ca_password=$(getProperty "ssl.ca.password" ${config_file})
ssl_country=$(getProperty "ssl.country" ${config_file})
ssl_state=$(getProperty "ssl.state" ${config_file})
ssl_locality=$(getProperty "ssl.locality" ${config_file})
ssl_organization=$(getProperty "ssl.organization" ${config_file})
ssl_organization_unit=$(getProperty "ssl.organization.unit" ${config_file})
ssl_company_domain=$(getProperty "ssl.company.domain" ${config_file})
ssl_p12_name=domainp12 # Not configurable

# Read composed property
ssl_subject_alt_names_array=$(getProperty "ssl.subject.alt.names" ${config_file})
ssl_subject_alt_names_list=${ssl_subject_alt_names_array[@]}
ssl_subject_alt_names=${ssl_subject_alt_names_list// /,}
ssl_subject_alt_names=${ssl_subject_alt_names//\$\{kiuwan_domain\}/$kiuwan_domain}
KIUWAN_PORT_FRONT1=$(getProperty "kiuwan.port.front1" $GLOBAL_CONFIG_PATH)
ssl_subject_alt_names=${ssl_subject_alt_names//\$\{kiuwan_port_front1\}/$KIUWAN_PORT_FRONT1}
KIUWAN_PORT_FRONT2=$(getProperty "kiuwan.port.front2" $GLOBAL_CONFIG_PATH)
ssl_subject_alt_names=${ssl_subject_alt_names//\$\{kiuwan_port_front2\}/$KIUWAN_PORT_FRONT2}
KIUWAN_PORT_MYSQL=$(getProperty "kiuwan.port.mysql" $GLOBAL_CONFIG_PATH)
ssl_subject_alt_names=${ssl_subject_alt_names//\$\{kiuwan_port_mysql\}/$KIUWAN_PORT_MYSQL}
KIUWAN_PORT_LOADBALANCER=$(getProperty "kiuwan.port.apache.https" $GLOBAL_CONFIG_PATH)
ssl_subject_alt_names=${ssl_subject_alt_names//\$\{kiuwan_port_loadbalancer\}/$KIUWAN_PORT_LOADBALANCER}

# Remove existing folder for the specified domain
if [ $CA_KEY_EXISTS == "true" ]
then
	echo Initializing "${kiuwan_domain}" directory
	rm -rf ${kiuwan_domain}
	mkdir "${kiuwan_domain}/"
fi

# Create a log file for this domain
log_file=${kiuwan_domain}/log.txt
date &>> ${log_file}

# Create a self-signed CA certificate if not exists
if [ -f "${ca_crt_file}" ] ; then
	echo Ignoring ${ca_crt_file} and ${ca_key_file} creation, ${ca_crt_file} file already exists. If you need to recreate the CA certificate, please manually remove the file "${ca_crt_file}"
else
	if [ $CA_KEY_EXISTS == "true" ]
	then
		log_file_ca=log.txt
		date &>> ${log_file_ca}
		echo Creating ${ca_key_file}
		openssl genrsa -des3 -passout pass:${ssl_ca_password} \
			-out ${ca_key_file} 4096 &>> ${log_file_ca}
		abortIfNotFound "${ca_key_file}"

		echo Creating self-signed ${ca_crt_file}
		openssl req -x509 -new -nodes -key ${ca_key_file} -sha256 \
			-days 2190 \
			-subj "/C=${ssl_country}/ST=${ssl_state}/L=${ssl_locality}/O=${ssl_organization}/OU=${ssl_organization_unit}/CN=${ssl_company_domain}" \
			-passin pass:${ssl_ca_password} \
			-out ${ca_crt_file} &>> ${log_file_ca}
		abortIfNotFound "${ca_crt_file}"
	fi
fi

abortIfNotFound "${ca_crt_file}"
if [ $CA_KEY_EXISTS == "true" ]
then
	abortIfNotFound "${ca_key_file}"
fi

# Copy the CA that will be used to sign the certificates to the current domain folder
echo Copying current CA to "${kiuwan_domain}"
cp -v ${ca_crt_file} ${kiuwan_domain}/ &>> ${log_file}

if [ $CA_KEY_EXISTS == "true" ]
then
	# Create the domain key file
	echo Creating ${key_file}
	openssl genrsa -out ${key_file} 4096 &>> ${log_file}
	abortIfNotFound "${key_file}"

	# Create the domain signing request
	echo Creating ${csr_file}
	openssl req -new -sha256 \
		-key ${key_file} \
		-subj "/C=${ssl_country}/ST=${ssl_state}/L=${ssl_locality}/O=${ssl_organization}/OU=${ssl_organization_unit}/CN=${kiuwan_domain}" \
		-reqexts SAN \
		-config <(cat /etc/ssl/openssl.cnf \
			<(printf "\n[SAN]\nsubjectAltName=${ssl_subject_alt_names}")) \
		-out ${csr_file} &>> ${log_file}
	abortIfNotFound "${csr_file}"

	# Create the domain certificate
	echo Creating ${crt_file}
	openssl x509 -req -in ${csr_file} -passin pass:${ssl_ca_password} \
		-days 730 \
		-CA ${ca_crt_file} -CAkey ${ca_key_file} -CAcreateserial \
		-out ${crt_file} -sha256 \
		-extensions SAN \
		-extfile <(cat /etc/ssl/openssl.cnf \
			<(printf "\n[SAN]\nsubjectAltName=${ssl_subject_alt_names}")) &>> ${log_file}
	abortIfNotFound "${crt_file}"
fi

# Create the domain p12 file
echo Creating ${p12_file}
openssl pkcs12 -export -chain -CAfile ${ca_crt_file} \
	-in ${crt_file} \
	-inkey ${key_file} \
	-out ${p12_file} -password pass:${java_keystore_password} \
	-name "${ssl_p12_name}" &>> ${log_file}
abortIfNotFound "${p12_file}"

# Create the Java keystore
echo Creating ${jks_file}
keytool -importkeystore -srckeystore ${p12_file} -srcstorepass ${java_keystore_password} \
	-srcalias ${ssl_p12_name} \
	-destkeystore ${jks_file} -deststorepass ${java_keystore_password} \
	-srcstoretype pkcs12 -destalias ${ssl_p12_name} &>> ${log_file}
abortIfNotFound "${jks_file}"

# Copy and change truststore password
echo Creating custom truststore with defined password: ${truststore_file}
cp ${default_truststore_file} ${truststore_file}
keytool -storepasswd -new ${java_truststore_password} -keystore ${truststore_file} -storepass changeit &>> ${log_file}

# Add the CA certificate to the Java keystore
echo Importing ${ca_crt_file} into ${jks_file}
keytool -import -noprompt -alias kiuwanca -keystore ${jks_file} -trustcacerts \
	-file ${ca_crt_file} -storepass ${java_keystore_password} &>> ${log_file}

# Add the CA certificate to the Java truststore
echo Importing ${ca_crt_file} into ${truststore_file}
keytool -import -noprompt -alias kiuwanca -keystore ${truststore_file} -trustcacerts \
	-file ${ca_crt_file} -storepass ${java_truststore_password} &>> ${log_file}

echo Done!
